@@ -1,4 +1,4 @@
-//@line 4 "/home/glazou/trees/official1.7/toolkit/components/url-classifier/nsUrlClassifierLib.js"
+//@line 4 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/nsUrlClassifierLib.js"
 
 // We wastefully reload the same JS files across components.  This puts all
 // the common JS files used by safebrowsing and url-classifier into a
@@ -10,7 +10,7 @@ const G_GDEBUG = false;
 
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 
-//@line 4 "/home/glazou/trees/official1.7/toolkit/components/url-classifier/content/moz/lang.js"
+//@line 4 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/content/moz/lang.js"
 
 
 /**
@@ -37,7 +37,7 @@ Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
  *
  * @returns {function} A partially-applied form of the speficied function.
  */
-function BindToObject(fn, self, opt_args) {
+this.BindToObject = function BindToObject(fn, self, opt_args) {
   var boundargs = fn.boundArgs_ || [];
   boundargs = boundargs.concat(Array.slice(arguments, 2, arguments.length));
 
@@ -90,7 +90,7 @@ Function.prototype.inherits = function(parentCtor) {
   this.superClass_ = parentCtor.prototype;
   this.prototype = new tempCtor();
 }
-//@line 4 "/home/glazou/trees/official1.7/toolkit/components/url-classifier/content/moz/preferences.js"
+//@line 4 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/content/moz/preferences.js"
 
 
 // Class for manipulating preferences. Aside from wrapping the pref
@@ -136,6 +136,7 @@ Function.prototype.inherits = function(parentCtor) {
  *
  * @constructor
  */
+this.G_Preferences =
 function G_Preferences(opt_startPoint, opt_getDefaultBranch) {
   this.debugZone = "prefs";
   this.observers_ = {};
@@ -286,6 +287,7 @@ G_Preferences.prototype.removeAllObservers = function() {
  * @constructor
  * @param callback Function to call when the preference changes
  */
+this.G_PreferenceObserver =
 function G_PreferenceObserver(callback) {
   this.debugZone = "prefobserver";
   this.callback_ = callback;
@@ -319,19 +321,43 @@ G_PreferenceObserver.prototype.QueryInterface = function(iid) {
   throw Components.results.NS_ERROR_NO_INTERFACE;
 }
 
-//@line 4 "/home/glazou/trees/official1.7/toolkit/components/url-classifier/content/moz/debug.js"
+//@line 4 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/content/moz/debug.js"
 
-//@line 834 "/home/glazou/trees/official1.7/toolkit/components/url-classifier/content/moz/debug.js"
+//@line 834 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/content/moz/debug.js"
 
 // Stubs for the debugging aids scattered through this component.
 // They will be expanded if you compile yourself a debug build.
 
-function G_Debug(who, msg) { }
-function G_Assert(who, condition, msg) { }
-function G_Error(who, msg) { }
-var G_debugService = { __noSuchMethod__: function() { } };
+this.G_Debug = function G_Debug(who, msg) { }
+this.G_Assert = function G_Assert(who, condition, msg) { }
+this.G_Error = function G_Error(who, msg) { }
+this.G_debugService = {
+    alsoDumpToConsole: () => {},
+    logFileIsEnabled: () => {},
+    enableLogFile: () => {},
+    disableLogFile: () => {},
+    getLogFile: () => {},
+    setLogFile: () => {},
+    enableDumpToConsole: () => {},
+    disableDumpToConsole: () => {},
+    getZone: () => {},
+    enableZone: () => {},
+    disableZone: () => {},
+    allZonesEnabled: () => {},
+    enableAllZones: () => {},
+    disableAllZones: () => {},
+    callTracingEnabled: () => {},
+    enableCallTracing: () => {},
+    disableCallTracing: () => {},
+    getLogFileErrorLevel: () => {},
+    setLogFileErrorLevel: () => {},
+    dump: () => {},
+    maybeDumpToFile: () => {},
+    observe: () => {},
+    reportScriptError_: () => {}
+};
 
-//@line 4 "/home/glazou/trees/official1.7/toolkit/components/url-classifier/content/moz/alarm.js"
+//@line 4 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/content/moz/alarm.js"
 
 
 // An Alarm fires a callback after a certain amount of time, or at
@@ -370,6 +396,7 @@ var G_debugService = { __noSuchMethod__: function() { } };
  * @param opt_maxTimes Number indicating a maximum number of times to 
  *                     repeat (obviously only useful when opt_repeating==true)
  */
+this.G_Alarm =
 function G_Alarm(callback, delayMS, opt_repeating, opt_maxTimes) {
   this.debugZone = "alarm";
   this.callback_ = callback;
@@ -455,11 +482,19 @@ G_Alarm.prototype.QueryInterface = function(iid) {
  *
  * For parameter documentation, see G_Alarm
  */
+this.G_ConditionalAlarm =
 function G_ConditionalAlarm(callback, delayMS, opt_repeating, opt_maxTimes) {
   G_Alarm.call(this, callback, delayMS, opt_repeating, opt_maxTimes);
   this.debugZone = "conditionalalarm";
 }
 
+G_ConditionalAlarm.inherits = function(parentCtor) {
+  var tempCtor = function(){};
+  tempCtor.prototype = parentCtor.prototype;
+  this.superClass_ = parentCtor.prototype;
+  this.prototype = new tempCtor();
+}
+
 G_ConditionalAlarm.inherits(G_Alarm);
 
 /**
@@ -477,7 +512,7 @@ G_ConditionalAlarm.prototype.notify = function(timer) {
     this.cancel();
   }
 }
-//@line 4 "/home/glazou/trees/official1.7/toolkit/components/url-classifier/content/moz/cryptohasher.js"
+//@line 4 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/content/moz/cryptohasher.js"
 
 
 // A very thin wrapper around nsICryptoHash. It's not strictly
@@ -497,6 +532,7 @@ G_ConditionalAlarm.prototype.notify = function(timer) {
 /**
  * Instantiate a new hasher. You must explicitly call init() before use!
  */
+this.G_CryptoHasher =
 function G_CryptoHasher() {
   this.debugZone = "cryptohasher";
   this.hasher_ = null;
@@ -619,7 +655,7 @@ G_CryptoHasher.prototype.toHex_ = function(str) {
   return hexrep.join('');
 }
 
-//@line 4 "/home/glazou/trees/official1.7/toolkit/components/url-classifier/content/moz/observer.js"
+//@line 4 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/content/moz/observer.js"
 
 
 // A couple of classes to simplify creating observers. 
@@ -650,7 +686,7 @@ G_CryptoHasher.prototype.toHex_ = function(str) {
  *
  * @constructor
  */
-function G_ObserverWrapper(topic, observeFunction) {
+this.G_ObserverWrapper = function G_ObserverWrapper(topic, observeFunction) {
   this.debugZone = "observer";
   this.topic_ = topic;
   this.observeFunction_ = observeFunction;
@@ -689,6 +725,7 @@ G_ObserverWrapper.prototype.observe = function(subject, topic, data) {
  *
  * @constructor
  */
+this.G_ObserverServiceObserver =
 function G_ObserverServiceObserver(topic, observeFunction, opt_onlyOnce) {
   this.debugZone = "observerserviceobserver";
   this.topic_ = topic;
@@ -719,7 +756,7 @@ G_ObserverServiceObserver.prototype.observe_ = function(subject, topic, data) {
     this.unregister();
 }
 
-//@line 4 "/home/glazou/trees/official1.7/toolkit/components/url-classifier/content/moz/protocol4.js"
+//@line 4 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/content/moz/protocol4.js"
 
 
 // A helper class that knows how to parse from and serialize to
@@ -740,7 +777,7 @@ G_ObserverServiceObserver.prototype.observe_ = function(subject, topic, data) {
  *
  * @constructor
  */
-function G_Protocol4Parser() {
+this.G_Protocol4Parser = function G_Protocol4Parser() {
   this.debugZone = "protocol4";
 
   this.protocol4RegExp_ = new RegExp("([^:]+):\\d+:(.*)$");
@@ -795,7 +832,7 @@ G_Protocol4Parser.prototype.serialize = function(map) {
   return text;
 }
 
-//@line 22 "/home/glazou/trees/official1.7/toolkit/components/url-classifier/nsUrlClassifierLib.js"
+//@line 22 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/nsUrlClassifierLib.js"
 
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -811,20 +848,21 @@ G_Protocol4Parser.prototype.serialize = function(map) {
 
 // HTTP responses that count as an error.  We also include any 5xx response
 // as an error.
-const HTTP_FOUND                 = 302;
-const HTTP_SEE_OTHER             = 303;
-const HTTP_TEMPORARY_REDIRECT    = 307;
+this.HTTP_FOUND                 = 302;
+this.HTTP_SEE_OTHER             = 303;
+this.HTTP_TEMPORARY_REDIRECT    = 307;
 
 /**
  * @param maxErrors Number of times to request before backing off.
  * @param retryIncrement Time (ms) for each retry before backing off.
  * @param maxRequests Number the number of requests needed to trigger backoff
  * @param requestPeriod Number time (ms) in which maxRequests have to occur to
- *     trigger the backoff behavior
+ *     trigger the backoff behavior (0 to disable maxRequests)
  * @param timeoutIncrement Number time (ms) the starting timeout period
  *     we double this time for consecutive errors
  * @param maxTimeout Number time (ms) maximum timeout period
  */
+this.RequestBackoff =
 function RequestBackoff(maxErrors, retryIncrement,
                         maxRequests, requestPeriod,
                         timeoutIncrement, maxTimeout) {
@@ -844,7 +882,7 @@ function RequestBackoff(maxErrors, retryIncrement,
 }
 
 /**
- * Reset the object for reuse.
+ * Reset the object for reuse. This deliberately doesn't clear requestTimes_.
  */
 RequestBackoff.prototype.reset = function() {
   this.numErrors_ = 0;
@@ -912,402 +950,7 @@ RequestBackoff.prototype.isErrorStatus = function(status) {
           HTTP_TEMPORARY_REDIRECT == status);
 }
 
-//@line 4 "/home/glazou/trees/official1.7/toolkit/components/url-classifier/content/url-crypto-key-manager.js"
-
-
-// This file implements the tricky business of managing the keys for our 
-// URL encryption. The protocol is:
-//
-// - Server generates secret key K_S
-// - Client starts up and requests a new key K_C from the server via HTTPS
-// - Server generates K_C and WrappedKey, which is K_C encrypted with K_S
-// - Server resonse with K_C and WrappedKey
-// - When client wants to encrypt a URL, it encrypts it with K_C and sends
-//   the encrypted URL along with WrappedKey
-// - Server decrypts WrappedKey with K_S to get K_C, and the URL with K_C
-//
-// This is, however, trickier than it sounds for two reasons. First,
-// we want to keep the number of HTTPS requests to an aboslute minimum
-// (like 1 or 2 per browser session). Second, the HTTPS request at
-// startup might fail, for example the user might be offline or a URL
-// fetch might need to be issued before the HTTPS request has
-// completed.
-//
-// We implement the following policy:
-// 
-// - Firefox will issue at most two HTTPS getkey requests per session
-// - Firefox will issue one HTTPS getkey request at startup if more than 24
-//   hours has passed since the last getkey request.
-// - Firefox will serialize to disk any key it gets
-// - Firefox will fall back on this serialized key until it has a
-//   fresh key
-// - The front-end can respond with a flag in a lookup request that tells
-//   the client to re-key. Firefox will issue a new HTTPS getkey request
-//   at this time if it has only issued one before
-
-// We store the user key in this file.  The key can be used to verify signed
-// server updates.
-const kKeyFilename = "urlclassifierkey3.txt";
-
-/**
- * A key manager for UrlCrypto. There should be exactly one of these
- * per appplication, and all UrlCrypto's should share it. This is
- * currently implemented by having the manager attach itself to the
- * UrlCrypto's prototype at startup. We could've opted for a global
- * instead, but I like this better, even though it is spooky action
- * at a distance.
- * XXX: Should be an XPCOM service
- *
- * @param opt_keyFilename String containing the name of the 
- *                        file we should serialize keys to/from. Used
- *                        mostly for testing.
- *
- * @param opt_testing Boolean indicating whether we are testing. If we 
- *                    are, then we skip trying to read the old key from
- *                    file and automatically trying to rekey; presumably
- *                    the tester will drive these manually.
- *
- * @constructor
- */
-function PROT_UrlCryptoKeyManager(opt_keyFilename, opt_testing) {
-  this.debugZone = "urlcryptokeymanager";
-  this.testing_ = !!opt_testing;
-  this.clientKey_ = null;          // Base64-encoded, as fetched from server
-  this.clientKeyArray_ = null;     // Base64-decoded into an array of numbers
-  this.wrappedKey_ = null;         // Opaque websafe base64-encoded server key
-  this.rekeyTries_ = 0;
-  this.updating_ = false;
-
-  // Don't do anything until keyUrl_ is set.
-  this.keyUrl_ = null;
-
-  this.keyFilename_ = opt_keyFilename ? 
-                      opt_keyFilename : kKeyFilename;
-
-  this.onNewKey_ = null;
-
-  // Convenience properties
-  this.MAX_REKEY_TRIES = PROT_UrlCryptoKeyManager.MAX_REKEY_TRIES;
-  this.CLIENT_KEY_NAME = PROT_UrlCryptoKeyManager.CLIENT_KEY_NAME;
-  this.WRAPPED_KEY_NAME = PROT_UrlCryptoKeyManager.WRAPPED_KEY_NAME;
-
-  if (!this.testing_) {
-    this.maybeLoadOldKey();
-  }
-}
-
-// Do ***** NOT ***** set this higher; HTTPS is expensive
-PROT_UrlCryptoKeyManager.MAX_REKEY_TRIES = 2;
-
-// Base pref for keeping track of when we updated our key.
-// We store the time as seconds since the epoch.
-PROT_UrlCryptoKeyManager.NEXT_REKEY_PREF = "urlclassifier.keyupdatetime.";
-
-// Once every 30 days (interval in seconds)
-PROT_UrlCryptoKeyManager.KEY_MIN_UPDATE_TIME = 30 * 24 * 60 * 60;
-
-// These are the names the server will respond with in protocol4 format
-PROT_UrlCryptoKeyManager.CLIENT_KEY_NAME = "clientkey";
-PROT_UrlCryptoKeyManager.WRAPPED_KEY_NAME = "wrappedkey";
-
-/**
- * Called to get ClientKey
- * @returns urlsafe-base64-encoded client key or null if we haven't gotten one.
- */
-PROT_UrlCryptoKeyManager.prototype.getClientKey = function() {
-  return this.clientKey_;
-}
-
-/**
- * Called by a UrlCrypto to get the current K_C
- *
- * @returns Array of numbers making up the client key or null if we 
- *          have no key
- */
-PROT_UrlCryptoKeyManager.prototype.getClientKeyArray = function() {
-  return this.clientKeyArray_;
-}
-
-/**
- * Called by a UrlCrypto to get WrappedKey
- *
- * @returns Opaque base64-encoded WrappedKey or null if we haven't
- *          gotten one
- */
-PROT_UrlCryptoKeyManager.prototype.getWrappedKey = function() {
-  return this.wrappedKey_;
-}
-
-/**
- * Change the key url.  When we do this, we go ahead and rekey.
- * @param keyUrl String
- */
-PROT_UrlCryptoKeyManager.prototype.setKeyUrl = function(keyUrl) {
-  // If it's the same key url, do nothing.
-  if (keyUrl == this.keyUrl_)
-    return;
-
-  this.keyUrl_ = keyUrl;
-  this.rekeyTries_ = 0;
-
-  // Check to see if we should make a new getkey request.
-  var prefs = new G_Preferences(PROT_UrlCryptoKeyManager.NEXT_REKEY_PREF);
-  var nextRekey = prefs.getPref(this.getPrefName_(this.keyUrl_), 0);
-  if (nextRekey < parseInt(Date.now() / 1000, 10)) {
-    this.reKey();
-  }
-}
-
-/**
- * Given a url, return the pref value to use (pref contains last update time).
- * We basically use the url up until query parameters.  This avoids duplicate
- * pref entries as version number changes over time.
- * @param url String getkey URL
- */
-PROT_UrlCryptoKeyManager.prototype.getPrefName_ = function(url) {
-  var queryParam = url.indexOf("?");
-  if (queryParam != -1) {
-    return url.substring(0, queryParam);
-  }
-  return url;
-}
-
-/**
- * Tell the manager to re-key. For safety, this method still obeys the
- * max-tries limit. Clients should generally use maybeReKey() if they
- * want to try a re-keying: it's an error to call reKey() after we've
- * hit max-tries, but not an error to call maybeReKey().
- */
-PROT_UrlCryptoKeyManager.prototype.reKey = function() {
-  if (this.updating_) {
-    G_Debug(this, "Already re-keying, ignoring this request");
-    return true;
-  }
-
-  if (this.rekeyTries_ > this.MAX_REKEY_TRIES)
-    throw new Error("Have already rekeyed " + this.rekeyTries_ + " times");
-
-  this.rekeyTries_++;
-
-  G_Debug(this, "Attempting to re-key");
-  // If the keyUrl isn't set, we don't do anything.
-  if (!this.testing_ && this.keyUrl_) {
-    this.fetcher_ = new PROT_XMLFetcher();
-    this.fetcher_.get(this.keyUrl_, BindToObject(this.onGetKeyResponse, this));
-    this.updating_ = true;
-
-    // Calculate the next time we're allowed to re-key.
-    var prefs = new G_Preferences(PROT_UrlCryptoKeyManager.NEXT_REKEY_PREF);
-    var nextRekey = parseInt(Date.now() / 1000, 10)
-                  + PROT_UrlCryptoKeyManager.KEY_MIN_UPDATE_TIME;
-    prefs.setPref(this.getPrefName_(this.keyUrl_), nextRekey);
-  }
-}
-
-/**
- * Try to re-key if we haven't already hit our limit. It's OK to call
- * this method multiple times, even if we've already tried to rekey
- * more than the max. It will simply refuse to do so.
- *
- * @returns Boolean indicating if it actually issued a rekey request (that
- *          is, if we haven' already hit the max)
- */
-PROT_UrlCryptoKeyManager.prototype.maybeReKey = function() {
-  if (this.rekeyTries_ > this.MAX_REKEY_TRIES) {
-    G_Debug(this, "Not re-keying; already at max");
-    return false;
-  }
-
-  this.reKey();
-  return true;
-}
-
-/**
- * Drop the existing set of keys.  Resets the rekeyTries variable to
- * allow a rekey to succeed.
- */
-PROT_UrlCryptoKeyManager.prototype.dropKey = function() {
-  this.rekeyTries_ = 0;
-  this.replaceKey_(null, null);
-}
-
-/**
- * @returns Boolean indicating if we have a key we can use 
- */
-PROT_UrlCryptoKeyManager.prototype.hasKey = function() {
-  return this.clientKey_ != null && this.wrappedKey_ != null;
-}
-
-PROT_UrlCryptoKeyManager.prototype.unUrlSafe = function(key)
-{
-    return key ? key.replace(/-/g, "+").replace(/_/g, "/") : "";
-}
-
-/**
- * Set a new key and serialize it to disk.
- *
- * @param clientKey String containing the base64-encoded client key 
- *                  we wish to use
- *
- * @param wrappedKey String containing the opaque base64-encoded WrappedKey
- *                   the server gave us (i.e., K_C encrypted with K_S)
- */
-PROT_UrlCryptoKeyManager.prototype.replaceKey_ = function(clientKey, 
-                                                          wrappedKey) {
-  if (this.clientKey_)
-    G_Debug(this, "Replacing " + this.clientKey_ + " with " + clientKey);
-
-  this.clientKey_ = clientKey;
-  this.clientKeyArray_ = Array.map(atob(this.unUrlSafe(clientKey)),
-                                   function(c) { return c.charCodeAt(0); });
-  this.wrappedKey_ = wrappedKey;
-
-  this.serializeKey_(this.clientKey_, this.wrappedKey_);
-
-  if (this.onNewKey_) {
-    this.onNewKey_();
-  }
-}
-
-/**
- * Try to write the key to disk so we can fall back on it. Fail
- * silently if we cannot. The keys are serialized in protocol4 format.
- *
- * @returns Boolean indicating whether we succeeded in serializing
- */
-PROT_UrlCryptoKeyManager.prototype.serializeKey_ = function() {
-
-  var map = {};
-  map[this.CLIENT_KEY_NAME] = this.clientKey_;
-  map[this.WRAPPED_KEY_NAME] = this.wrappedKey_;
-  
-  try {  
-
-    var keyfile = Cc["@mozilla.org/file/directory_service;1"]
-                 .getService(Ci.nsIProperties)
-                 .get("ProfD", Ci.nsILocalFile); /* profile directory */
-    keyfile.append(this.keyFilename_);
-
-    if (!this.clientKey_ || !this.wrappedKey_) {
-      keyfile.remove(true);
-      return;
-    }
-
-    var data = (new G_Protocol4Parser()).serialize(map);
-
-    try {
-      var stream = Cc["@mozilla.org/network/file-output-stream;1"]
-                   .createInstance(Ci.nsIFileOutputStream);
-      stream.init(keyfile,
-                  0x02 | 0x08 | 0x20 /* PR_WRONLY | PR_CREATE_FILE | PR_TRUNCATE */,
-                  -1 /* default perms */, 0 /* no special behavior */);
-      stream.write(data, data.length);
-    } finally {
-      stream.close();
-    }
-    return true;
-
-  } catch(e) {
-
-    G_Error(this, "Failed to serialize new key: " + e);
-    return false;
-
-  }
-}
-
-/**
- * Invoked when we've received a protocol4 response to our getkey
- * request. Try to parse it and set this key as the new one if we can.
- *
- *  @param responseText String containing the protocol4 getkey response
- */ 
-PROT_UrlCryptoKeyManager.prototype.onGetKeyResponse = function(responseText) {
-
-  var response = (new G_Protocol4Parser).parse(responseText);
-  var clientKey = response[this.CLIENT_KEY_NAME];
-  var wrappedKey = response[this.WRAPPED_KEY_NAME];
-
-  this.updating_ = false;
-  this.fetcher_ = null;
-
-  if (response && clientKey && wrappedKey) {
-    G_Debug(this, "Got new key from: " + responseText);
-    this.replaceKey_(clientKey, wrappedKey);
-  } else {
-    G_Debug(this, "Not a valid response for /newkey");
-  }
-}
-
-/**
- * Set the callback to be called whenever we get a new key.
- *
- * @param callback The callback.
- */
-PROT_UrlCryptoKeyManager.prototype.onNewKey = function(callback) 
-{
-  this.onNewKey_ = callback;
-}
-
-/**
- * Attempt to read a key we've previously serialized from disk, so
- * that we can fall back on it in case we can't get one from the
- * server. If we get a key, only use it if we don't already have one
- * (i.e., if our startup HTTPS request died or hasn't yet completed).
- *
- * This method should be invoked early, like when the user's profile
- * becomes available.
- */ 
-PROT_UrlCryptoKeyManager.prototype.maybeLoadOldKey = function() {
-  
-  var oldKey = null;
-  try {  
-    var keyfile = Cc["@mozilla.org/file/directory_service;1"]
-                 .getService(Ci.nsIProperties)
-                 .get("ProfD", Ci.nsILocalFile); /* profile directory */
-    keyfile.append(this.keyFilename_);
-    if (keyfile.exists()) {
-      try {
-        var fis = Cc["@mozilla.org/network/file-input-stream;1"]
-                  .createInstance(Ci.nsIFileInputStream);
-        fis.init(keyfile, 0x01 /* PR_RDONLY */, 0444, 0);
-        var stream = Cc["@mozilla.org/scriptableinputstream;1"]
-                     .createInstance(Ci.nsIScriptableInputStream);
-        stream.init(fis);
-        oldKey = stream.read(stream.available());
-      } finally {
-        if (stream)
-          stream.close();
-      }
-    }
-  } catch(e) {
-    G_Debug(this, "Caught " + e + " trying to read keyfile");
-    return;
-  }
-   
-  if (!oldKey) {
-    G_Debug(this, "Couldn't find old key.");
-    return;
-  }
-
-  oldKey = (new G_Protocol4Parser).parse(oldKey);
-  var clientKey = oldKey[this.CLIENT_KEY_NAME];
-  var wrappedKey = oldKey[this.WRAPPED_KEY_NAME];
-
-  if (oldKey && clientKey && wrappedKey && !this.hasKey()) {
-    G_Debug(this, "Read old key from disk.");
-    this.replaceKey_(clientKey, wrappedKey);
-  }
-}
-
-PROT_UrlCryptoKeyManager.prototype.shutdown = function() {
-  if (this.fetcher_) {
-    this.fetcher_.cancel();
-    this.fetcher_ = null;
-  }
-}
-
-
-//@line 4 "/home/glazou/trees/official1.7/toolkit/components/url-classifier/content/xml-fetcher.js"
+//@line 4 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/content/xml-fetcher.js"
 
 // A simple class that encapsulates a request. You'll notice the
 // style here is different from the rest of the extension; that's
@@ -1315,9 +958,6 @@ PROT_UrlCryptoKeyManager.prototype.shutdown = function() {
 // point it might be nice to replace this with something better
 // (e.g., something that has explicit onerror handler, ability
 // to set headers, and so on).
-//
-// The only interesting thing here is its ability to strip cookies
-// from the request.
 
 /**
  * Because we might be in a component, we can't just assume that
@@ -1326,7 +966,7 @@ PROT_UrlCryptoKeyManager.prototype.shutdown = function() {
  *
  * @return XMLHttpRequest object
  */
-function PROT_NewXMLHttpRequest() {
+this.PROT_NewXMLHttpRequest = function PROT_NewXMLHttpRequest() {
   var Cc = Components.classes;
   var Ci = Components.interfaces;
   var request = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
@@ -1341,15 +981,20 @@ function PROT_NewXMLHttpRequest() {
  * the content it receives. Asynchronous, so uses a closure for the
  * callback.
  *
- * @param opt_stripCookies Boolean indicating whether we should strip
- *                         cookies from this request
- * 
+ * Note, that XMLFetcher is only used for SafeBrowsing, therefore
+ * we inherit from nsILoadContext, so we can use the callbacks on the
+ * channel to separate the safebrowsing cookie based on a reserved
+ * appId.
  * @constructor
  */
-function PROT_XMLFetcher(opt_stripCookies) {
+this.PROT_XMLFetcher = function PROT_XMLFetcher() {
   this.debugZone = "xmlfetcher";
   this._request = PROT_NewXMLHttpRequest();
-  this._stripCookies = !!opt_stripCookies;
+  // implements nsILoadContext
+  this.appId = Ci.nsIScriptSecurityManager.SAFEBROWSING_APP_ID;
+  this.isInBrowserElement = false;
+  this.usePrivateBrowsing = false;
+  this.isContent = false;
 }
 
 PROT_XMLFetcher.prototype = {
@@ -1373,9 +1018,6 @@ PROT_XMLFetcher.prototype = {
     this._request.open("GET", page, asynchronous);
     this._request.channel.notificationCallbacks = this;
 
-    if (this._stripCookies)
-      new PROT_CookieStripper(this._request.channel);
-
     // Create a closure
     var self = this;
     this._request.addEventListener("readystatechange", function() {
@@ -1424,77 +1066,11 @@ PROT_XMLFetcher.prototype = {
     return this.QueryInterface(iid);
   },
 
-  QueryInterface: function(iid) {
-    if (!iid.equals(Components.interfaces.nsIInterfaceRequestor) &&
-        !iid.equals(Components.interfaces.nsISupports))
-      throw Components.results.NS_ERROR_NO_INTERFACE;
-    return this;
-  }
+  QueryInterface: XPCOMUtils.generateQI([Ci.nsIInterfaceRequestor,
+                                         Ci.nsISupports,
+                                         Ci.nsILoadContext])
 };
-
-
-/**
- * This class knows how to strip cookies from an HTTP request. It
- * listens for http-on-modify-request, and modifies the request
- * accordingly. We can't do this using xmlhttprequest.setHeader() or
- * nsIChannel.setRequestHeader() before send()ing because the cookie
- * service is called after send().
- * 
- * @param channel nsIChannel in which the request is happening
- * @constructor
- */
-function PROT_CookieStripper(channel) {
-  this.debugZone = "cookiestripper";
-  this.topic_ = "http-on-modify-request";
-  this.channel_ = channel;
-
-  var Cc = Components.classes;
-  var Ci = Components.interfaces;
-  this.observerService_ = Cc["@mozilla.org/observer-service;1"]
-                          .getService(Ci.nsIObserverService);
-  this.observerService_.addObserver(this, this.topic_, false);
-
-  // If the request doesn't issue, don't hang around forever
-  var twentySeconds = 20 * 1000;
-  this.alarm_ = new G_Alarm(BindToObject(this.stopObserving, this), 
-                            twentySeconds);
-}
-
-/**
- * Invoked by the observerservice. See nsIObserve.
- */
-PROT_CookieStripper.prototype.observe = function(subject, topic, data) {
-  if (topic != this.topic_ || subject != this.channel_)
-    return;
-
-  G_Debug(this, "Stripping cookies for channel.");
-
-  this.channel_.QueryInterface(Components.interfaces.nsIHttpChannel);
-  this.channel_.setRequestHeader("Cookie", "", false /* replace, not add */);
-  this.alarm_.cancel();
-  this.stopObserving();
-}
-
-/**
- * Remove us from the observerservice
- */
-PROT_CookieStripper.prototype.stopObserving = function() {
-  G_Debug(this, "Removing observer");
-  this.observerService_.removeObserver(this, this.topic_);
-  this.channel_ = this.alarm_ = this.observerService_ = null;
-}
-
-/**
- * XPCOM cruft
- */
-PROT_CookieStripper.prototype.QueryInterface = function(iid) {
-  var Ci = Components.interfaces;
-  if (iid.equals(Ci.nsISupports) || iid.equals(Ci.nsIObserve))
-    return this;
-  throw Components.results.NS_ERROR_NO_INTERFACE;
-}
-
-//@line 26 "/home/glazou/trees/official1.7/toolkit/components/url-classifier/nsUrlClassifierLib.js"
+//@line 25 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/nsUrlClassifierLib.js"
 
 // Expose this whole component.
 var lib = this;
